import { useReducer } from 'react';
function createReducer(hooks = {}) {
    return (store, action) => {
        try {
            const { type, payload } = action;
            switch (type) {
                case 'reset':
                    return payload;
                case 'update': {
                    for (const k in payload) {
                        const key = k;
                        const value = payload[key];
                        const hook = hooks[key];
                        payload[key] = (typeof hook === 'function')
                            ? hook(value) ?? value
                            : value;
                    }
                    return { ...store, ...payload };
                }
                case 'write': {
                    return { ...store, ...payload };
                }
                default:
                    throw new Error(`Invalid action: ${String(type)}`);
            }
        }
        catch (err) {
            console.error(err);
            const { message } = err;
            return { ...store, error: message };
        }
    };
}
export function useStore(defaults, hooks) {
    const reducer = createReducer(hooks);
    const [store, dispatch] = useReducer(reducer, defaults);
    function setError(err) {
        const error = (err instanceof Error)
            ? err.message
            : err;
        dispatch({ type: 'write', payload: { error } });
    }
    function reset(store) {
        const payload = { ...defaults, ...store };
        dispatch({ type: 'reset', payload });
    }
    function update(store) {
        dispatch({ type: 'update', payload: store });
    }
    return {
        reset,
        setError,
        store,
        update
    };
}
//# sourceMappingURL=useStore.js.map